VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cNTFS"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'http://support.microsoft.com/kb/295004 (among other sources used to write this class.)

'-- object constants. Only dealing with registry keys and folders here.
Private Const SE_FILE_OBJECT As Long = 1
Private Const SE_REG_KEY As Long = 4

Private Const DACL_SECURITY_INFORMATION As Long = 4 'constant to return DACL pointer.
  '-- constants for setting or revoking access.
Private Const SET_ACCESS As Long = 2
Private Const REVOKE_ACCESS As Long = 4

'-- constants for AllocateAndInitializeSID
Private Const SECURITY_BUILTIN_DOMAIN_RID    As Long = &H20
Private Const DOMAIN_ALIAS_RID_ADMINS        As Long = &H220
Private Const DOMAIN_ALIAS_RID_USERS         As Long = &H221
Private Const SECURITY_WORLD_RID             As Long = 0
Private Const SECURITY_WORLD_SID_AUTHORITY   As Long = 1
Private Const SECURITY_NT_AUTHORITY          As Long = 5

Private Type SID_IDENTIFIER_AUTHORITY
   Value(6) As Byte
End Type

' Standard access rights extracted from WinNT.h
Private Const SYNCHRONIZE = &H100000
Private Const READ_CONTROL = &H20000
Private Const WRITE_DAC = &H40000
Private Const WRITE_OWNER = &H80000
Private Const STANDARD_RIGHTS_DELETE = &H10000
Private Const STANDARD_RIGHTS_REQUIRED = &HF0000
Private Const SPECIFIC_RIGHTS_REQUIRED = &H1FF
Private Const STANDARD_RIGHTS_ALL = &H1F0000
Private Const SPECIFIC_RIGHTS_ALL = &HFFFF&

'specific flags
Private Const FILE_READ_DATA = &H1& ' file & pipe
Private Const FILE_LIST_DIRECTORY = &H1& ' directory    '1st byte from right.

Private Const FILE_WRITE_DATA = &H2& ' file & pipe
Private Const FILE_ADD_FILE = &H2& ' directory              '2nd

Private Const FILE_APPEND_DATA = &H4& ' file
Private Const FILE_ADD_SUBDIRECTORY = &H4& ' directory
Private Const FILE_CREATE_PIPE_INSTANCE = &H4& ' named pipe    '3rd

Private Const FILE_READ_EA = &H8& ' file & directory  '   4th
Private Const FILE_WRITE_EA = &H10& ' file & directory    '5th
Private Const FILE_EXECUTE = &H20& ' file
Private Const FILE_TRAVERSE = &H20& ' directory       '6th
Private Const FILE_DELETE_CHILD = &H40& ' directory      '7th
Private Const FILE_READ_ATTRIBUTES = &H80& ' all        '8th
Private Const FILE_WRITE_ATTRIBUTES = &H100& ' all     '9th

'-- Constants designed to work with both folders and reg. keys.
Private Const PERMS_WRITE As Long = FILE_WRITE_DATA Or FILE_APPEND_DATA Or READ_CONTROL
Private Const PERMS_READ As Long = FILE_READ_DATA Or READ_CONTROL
Private Const PERMS_EXECUTE As Long = FILE_EXECUTE Or FILE_READ_ATTRIBUTES Or READ_CONTROL
Private Const PERMS_DELETE As Long = STANDARD_RIGHTS_DELETE
Private Const PERMS_ALL As Long = STANDARD_RIGHTS_REQUIRED Or SPECIFIC_RIGHTS_REQUIRED
   ' NOTE: PERMS_ALL is first 9 flags of specific(seem to be the only ones used) and all standard rights
   ' except SYNCHRONIZE. That flag is never used with the Registry, so it's left out here for compat.
  
' Inheritance Flags
Private Const CONTAINER_INHERIT_ACE = &H2
Private Const OBJECT_INHERIT_ACE = &H1
Private Const NO_PROPAGATE_INHERIT_ACE = &H4

Enum TRUSTEE_FORM
    TRUSTEE_IS_SID
    TRUSTEE_IS_NAME
End Enum

Enum TRUSTEE_TYPE
    TRUSTEE_IS_UNKNOWN
    TRUSTEE_IS_USER
    TRUSTEE_IS_GROUP
End Enum

Public Enum NTPerms
  NTPERMS_ERROR = -1
  NTPERMS_SUCCESS = 0
  NTPERMS_READ = 1
  NTPERMS_WRITE = 2
  NTPERMS_DELETE = 4
  NTPERMS_REG_ALL = 7
  NTPERMS_EXECUTE = 8
  NTPERMS_ALL = 16
End Enum

Private Type TRUSTEE
    pMultipleTrustee As Long
    MultipleTrusteeOperation As Long
    TrusteeForm As TRUSTEE_FORM
    TrusteeType As TRUSTEE_TYPE
    ptstrName As String
End Type

Private Type EXPLICIT_ACCESS
    grfAccessPermissions As Long
    grfAccessMode As Long
    grfInheritance As Long
    pTrustee As TRUSTEE
End Type

' High Level access control API declarations
Private Declare Sub BuildExplicitAccessWithName Lib "advapi32.dll" Alias "BuildExplicitAccessWithNameA" (EA As Any, ByVal TrusteeName As String, ByVal AccessPermissions As Long, ByVal AccessMode As Integer, ByVal Inheritance As Long)
Private Declare Function SetEntriesInAcl Lib "advapi32.dll" Alias "SetEntriesInAclA" (ByVal CountofExplicitEntries As Long, EA As Any, ByVal OldAcl As Long, NewAcl As Long) As Long
Private Declare Function GetNamedSecurityInfo Lib "advapi32.dll" Alias "GetNamedSecurityInfoA" (ByVal ObjName As String, ByVal SE_OBJECT_TYPE As Long, ByVal SecInfo As Long, ByVal pSid As Long, ByVal pSidGroup As Long, pDacl As Long, ByVal pSacl As Long, pSecurityDescriptor As Long) As Long
Private Declare Function SetNamedSecurityInfo Lib "advapi32.dll" Alias "SetNamedSecurityInfoA" (ByVal ObjName As String, ByVal SE_OBJECT As Long, ByVal SecInfo As Long, ByVal pSid As Long, ByVal pSidGroup As Long, ByVal pDacl As Long, ByVal pSacl As Long) As Long
Private Declare Function LocalFree Lib "KERNEL32" (ByVal hMem As Long) As Long
Private Declare Function GetEffectiveRightsFromAcl Lib "advapi32.dll" Alias "GetEffectiveRightsFromAclA" (ByVal pAcl As Long, ByRef pTrustee As TRUSTEE, ByRef pAccessRights As Long) As Long

' functions to get name from SID:
Private Declare Function LookupAccountSid Lib "advapi32.dll" Alias "LookupAccountSidA" (ByVal lpSystemName As String, ByVal Sid As Long, ByVal name As String, cbName As Long, ByVal ReferencedDomainName As String, cbReferencedDomainName As Long, peUse As Long) As Long
Private Declare Function AllocateAndInitializeSid Lib "advapi32.dll" (pIdentifierAuthority As SID_IDENTIFIER_AUTHORITY, ByVal nSubAuthorityCount As Byte, ByVal nSubAuthority0 As Long, ByVal nSubAuthority1 As Long, ByVal nSubAuthority2 As Long, ByVal nSubAuthority3 As Long, ByVal nSubAuthority4 As Long, ByVal nSubAuthority5 As Long, ByVal nSubAuthority6 As Long, ByVal nSubAuthority7 As Long, lpPSid As Long) As Long
Private Declare Sub FreeSid Lib "advapi32.dll" (pSid As Any)

Private Declare Function GetVolumeInformation Lib "KERNEL32" Alias "GetVolumeInformationA" (ByVal lpRootPathName As String, ByVal lpVolumeNameBuffer As String, ByVal nVolumeNameSize As Long, lpVolumeSerialNumber As Long, lpMaximumComponentLength As Long, lpFileSystemFlags As Long, ByVal lpFileSystemNameBuffer As String, ByVal nFileSystemNameSize As Long) As Long
Private Declare Function GetUserName Lib "advapi32.dll" Alias "GetUserNameA" (ByVal lpBuffer As String, nSize As Long) As Long

'------------------ temp testing code-------------------
Private PVal As Long  '--temp

Public Property Get ACCRights() As Long  '--temp
 ACCRights = PVal
End Property '--temp


'------------- end temp code ------------------------

'-================= CLASS METHODS: ======================================

' * Function IsNTFS()  Boolean. Is the disk NTFS formatted?

 '----------------------------------------------------------
 
' * Function GetPermissions(sPath As String, LType As Long, Who As Long, LErrCode As Long) As NTPerms
 '  sPath: path of folder, file, or Reg. key:  C:\folder1  MACHINE\Software\Mozilla  CLASSES_ROOT\.txt  CURRENT_USER\Software\Mozilla
 '  LType: 1 - folder/file.  non-1 - Registry key.
 '  Who:  1-Current user. 2-Administrators. 3-Users.
 '  LErrCode: Returns error info. with errors from API calls.
    ' typical error: 87-bad parameter. (Ex. folder path sent and LType <> 1)
  '  Function return:
 '   -1 - error. 0-no permissions. If > 0 then value is a combination of the following flags:
 '    1-R. 2-W. 4-D. 8-X. 16-All.
 '     With folders the return is generally RWDXA or R. But it can be different. For example,
 '         an Admin may have WE or WDE permissions with a user settings folder they are not allowed to view.
 '     With Reg. keys it's usually RWD or R. With Registry, the ALL flag is never set.

'-----------------------------------------------------------

' * Function SetPermissions(sPath As String, LType As Long, Who As Long, Inherit As Boolean, ClearDACL As Boolean, RevokeRights As Boolean, LErrCode As Long) As Long
 '  sPath: path of folder, file, or Reg. key:  C:\folder1  MACHINE\Software\Mozilla  CLASSES_ROOT\.txt  CURRENT_USER\Software\Mozilla
 '  LType: 1 - folder/file.  non-1 - Registry key.
 '  Who:  1-Current user. 2-Administrators. 3-Users.
 '  Inherit: True: Permissions extend to subfolders/subkeys. False: Permission for named folder/key only.
 '  ClearDACL: True to clear all pre-existing ACEs for named trustee. This SHOULD BE TRUE unless there is
'     a strong reason not to do so. Setting/revoking permissions may not always work otherwise because
'     a pre-existing ACE in the DACL could override the changes made by SetPermissions.
'   RevokeRights: True: Revokes all custom rights (ACEs) for named trustee. False: used to set rights.
' LErrCode: Error code returned from API calls.
' typical errors: 5-Access Denied. (not admin or under UAC?) 87-bad parameter. (Ex. folder path sent and LType <> 1)
'  Function return: 0-success. -1-unknown error. 1-GetNamedSecurityInfo error. 2-SetEntriesInACL error. 3-SetNamedSecurityInfo error.

'====================================================================================

Public Function IsNTFS() As Boolean
  Dim LRet As Long, LFlags As Long, LBuf As Long, Pt1 As Long
  Dim sBuf As String, sBufFS As String, sSysName As String
     On Error Resume Next
   LBuf = 256
   sBuf = String$(LBuf, 0)
   sBufFS = String$(LBuf, 0)
   LRet = GetVolumeInformation(vbNullString, sBuf, LBuf, 0&, 0&, LFlags, sBufFS, LBuf)
     If (LRet <> 0) Then
       Pt1 = InStr(1, sBufFS, Chr(0), vbBinaryCompare)
       If Pt1 > 1 Then sSysName = Left$(sBufFS, (Pt1 - 1))
     End If
  If UCase$(sSysName) = "NTFS" Then IsNTFS = True
End Function
   
'-- takes registry path or folder path, LType: 1-folder or file. non1-Registry key.  Who: trustee code (1-curuser, 2-administrators, 3-Users)
'-- returns 0 on success. -1 error. 1 or 2, API error and LErrcode holds error code.
'-- if return is 0 then sPerms holds permssions string like: RWEA, R-E-, etc.
Public Function GetPermissions(sPath As String, LType As Long, Who As Long, LErrCode As Long) As NTPerms
  Dim LRet As Long, LDesc As Long, CurDACL As Long, LObjType As Long, CurPerms As Long, Flags As Long
  Dim sUserName As String
  Dim TR As TRUSTEE
     GetPermissions = NTPERMS_ERROR
      On Error GoTo Woops
        
   If (LType = 1) Then
     LObjType = SE_FILE_OBJECT 'type 1
   Else
     LObjType = SE_REG_KEY     'type 4
   End If
     
    sUserName = GetLocalName(Who)
   LRet = GetNamedSecurityInfo(sPath, LObjType, DACL_SECURITY_INFORMATION, 0&, 0&, CurDACL, 0&, LDesc)
     If LRet <> 0 Then GetPermissions = NTPERMS_ERROR: LErrCode = LRet: GoTo Woops
     
   With TR
     .TrusteeForm = TRUSTEE_IS_NAME
     .TrusteeType = TRUSTEE_IS_UNKNOWN
     .ptstrName = sUserName
   End With
   
   LRet = GetEffectiveRightsFromAcl(CurDACL, TR, CurPerms)
     If LRet <> 0 Then GetPermissions = NTPERMS_ERROR: LErrCode = LRet:: GoTo Woops
     
   PVal = CurPerms  '--temp for testing.
   
  If (CurPerms And PERMS_READ) = PERMS_READ Then GetPermissions = NTPERMS_READ
  If (CurPerms And PERMS_WRITE) = PERMS_WRITE Then GetPermissions = GetPermissions Or NTPERMS_WRITE
  If (CurPerms And PERMS_DELETE) = PERMS_DELETE Then GetPermissions = GetPermissions Or NTPERMS_DELETE
  If (CurPerms And PERMS_EXECUTE) = PERMS_EXECUTE Then GetPermissions = GetPermissions Or NTPERMS_EXECUTE
  If (CurPerms And PERMS_ALL) = PERMS_ALL Then GetPermissions = GetPermissions Or NTPERMS_ALL

   'GetPermissions = NTPERMS_SUCCESS  'success.
Woops:
  If LDesc <> 0 Then LocalFree LDesc
End Function

 ' set permissions. ' sPath is reg. key or folder path. LType is 1 for folders, non-1 for reg.
 ' Who: 1-currentuser. 2-admin. 3-Users. ClearDACL: True to create a new DACL. RemoveRights: True to remove any ACEs for
 ' trustee, undoing any custom permissions set earlier. LErrcode: API error number if any.
Public Function SetPermissions(sPath As String, LType As Long, Who As Long, Inherit As Boolean, ClearDACL As Boolean, RevokeRights As Boolean, LErrCode As Long) As Long
  Dim LRet As Long, LDesc As Long, NewDACL As Long, OldDACL As Long, LObjType As Long
  Dim LInherit As Long
  Dim sUserName As String
  Dim EA As EXPLICIT_ACCESS
     SetPermissions = -1 '-- error other than API error.
      On Error GoTo Woops
   If (LType = 1) Then
     LObjType = SE_FILE_OBJECT 'type 1  full path to file or folder.
   Else
     LObjType = SE_REG_KEY     'type 4  registry key: "CLASSES_ROOT\somepath", "CURRENT_USER\somepath", "MACHINE\somepath", or "USERS\somepath"
   End If
   
     '-- do subfolders/subkeys inherit this permission?
   If Inherit = True Then
     LInherit = CONTAINER_INHERIT_ACE Or OBJECT_INHERIT_ACE
   Else
     LInherit = NO_PROPAGATE_INHERIT_ACE
   End If
   
     '-- Get the existing DACL pointer. (Not actually needed if clearing DACL.)
   LRet = GetNamedSecurityInfo(sPath, LObjType, DACL_SECURITY_INFORMATION, 0&, 0&, OldDACL, 0&, LDesc)
     If LRet <> 0 Then SetPermissions = 1: LErrCode = LRet: GoTo Woops
     
     sUserName = GetLocalName(Who)  '-- get language-specific name of 1-CurUser, 2-Admnistrators, or 3-Users.
     
       '-- Setting new permissions or removing all custom permssions set previously?
   If (RevokeRights = True) Then
     BuildExplicitAccessWithName EA, sUserName, 0&, REVOKE_ACCESS, LInherit
   Else
     BuildExplicitAccessWithName EA, sUserName, SPECIFIC_RIGHTS_ALL Or STANDARD_RIGHTS_ALL, SET_ACCESS, LInherit
   End If
    
      If ClearDACL = True Then OldDACL = 0
    LRet = SetEntriesInAcl(1, EA, OldDACL, NewDACL)
      If LRet <> 0 Then SetPermissions = 2: LErrCode = LRet: GoTo Woops
      
    LRet = SetNamedSecurityInfo(sPath, LObjType, DACL_SECURITY_INFORMATION, 0&, 0&, NewDACL, 0&)
        If LRet <> 0 Then SetPermissions = 3: LErrCode = LRet: GoTo Woops
        
    SetPermissions = 0
Woops:
  On Error Resume Next
  If NewDACL <> 0 Then LocalFree NewDACL
  If LDesc <> 0 Then LocalFree LDesc
End Function

  '-- Function to return name of group for setting permissions, regardless of default language on system.
  '-- The call to BuildExplicitAccessWithName uses a name string for the second parameter.
  '-- One can use "Everyone", Users", etc. on an English system but that fails with other languages.
  '-- Entity: 1-current user. 2-Administrators. 3-everyone.
 Private Function GetLocalName(Entity As Long) As String
   Dim LRet As Long, LenName As Long, LenDomName As Long, LType As Long, Pt1 As Long, LSid As Long
   Dim SName As String, sDomName As String
   Dim SIA As SID_IDENTIFIER_AUTHORITY
      On Error Resume Next
   GetLocalName = ""
   
  If (Entity <> 2) And (Entity <> 3) Then  ' default to current user.
     Pt1 = 255
     SName = String$(Pt1, 0)
     LRet = GetUserName(SName, Pt1)
    If (LRet <> 0) Then GetLocalName = Left$(SName, Pt1 - 1)
    Exit Function
  End If
  
  If Entity = 2 Then  'admins
      SIA.Value(5) = SECURITY_NT_AUTHORITY
      LRet = AllocateAndInitializeSid(SIA, 2, SECURITY_BUILTIN_DOMAIN_RID, DOMAIN_ALIAS_RID_ADMINS, 0, 0, 0, 0, 0, 0, LSid)
  ElseIf Entity = 3 Then 'users
       SIA.Value(5) = SECURITY_NT_AUTHORITY
       LRet = AllocateAndInitializeSid(SIA, 2, SECURITY_BUILTIN_DOMAIN_RID, DOMAIN_ALIAS_RID_USERS, 0, 0, 0, 0, 0, 0, LSid)
    ' everyone. doesn't seem to work.
    '  SIA.Value(5) = SECURITY_WORLD_SID_AUTHORITY
    '  LRet = AllocateAndInitializeSid(SIA, 1, SECURITY_WORLD_RID, 0, 0, 0, 0, 0, 0, 0, LSid)
  End If
   If LRet = 1 Then
      SName = Space$(255)
      sDomName = Space$(255)
      LenName = 255
      LenDomName = 255
     LRet = LookupAccountSid(vbNullString, LSid, SName, LenName, sDomName, LenDomName, LType)
      If LRet = 1 Then
        Pt1 = InStr(SName, Chr(0))
         If Pt1 > 0 Then SName = Left(SName, (Pt1 - 1))
        Pt1 = InStr(SName, "\")
         If Pt1 > 0 Then SName = Right(SName, (Len(SName) - Pt1))
        GetLocalName = SName
      End If
        FreeSid ByVal LSid
   End If
End Function

