Attribute VB_Name = "ModOps"
Option Explicit
'------------------------------------------------------
'--------   ModOps contains central methods for setup such as
'--------   copying install files.
'------------------------------------------------------------

  '-- called from Mod1, GetPathStrings. reads entries in Setup.lst file.
Public Function ReadIniFile(ByVal strIniFile As String, ByVal strSection As String, ByVal strKey As String) As String
 Dim strBuffer As String
  strBuffer = String$(MAX_PATH, 0)
    If GetPrivateProfileString(strSection, strKey, vbNullString, strBuffer, MAX_PATH, strIniFile) Then
      ReadIniFile = StringFromBuffer(strBuffer)
    End If
End Function

Public Function GetAppRemovalCmdLine(ByVal strAppRemovalEXE As String, ByVal strAppRemovalLog, ByVal nErrorLevel As Integer, Optional fWaitForParent As Boolean = False)

    Dim strEXE As String
    Dim strLog As String
    Dim strErrLevel As String
    Dim strForce As String
    Dim strWait As String
   
    Dim curProcessId As Currency
    Dim Wrap As Currency
    Dim lProcessId As Long
    Dim cProcessId As Currency

    strEXE = AddQuotesToFN(strAppRemovalEXE)
    strLog = "-n " & QUOTEMARK & GetLongPath(strAppRemovalLog) & QUOTEMARK
    

    If (nErrorLevel <> APPREMERR_NONE) Then
        strErrLevel = "-e " & Format$(nErrorLevel)
        strForce = " -f"
    End If
    If fWaitForParent Then
        Wrap = 2 * (CCur(&H7FFFFFFF) + 1)

        'Always print as an unsigned long
        lProcessId = GetCurrentProcessId()
        cProcessId = lProcessId
        If cProcessId < 0 Then cProcessId = cProcessId + Wrap

        strWait = " -w " & str$(cProcessId)
    End If

    GetAppRemovalCmdLine = strEXE & " " & strLog & " " & strErrLevel & strForce & strWait
End Function


' FUNCTION: CopyFile
'
' Uses the Windows VerInstallFile API to copy a file from
' the specified source location/name to the destination
' location/name.
' If the file is successfully updated and the file is a
' shared file (fShared = True), then the
' files reference count is updated (32-bits only)
'
' IN: [strSrcDir] - directory where source file is located
'     [strDestDir] - destination directory for file
'     [strSrcName] - name of source file
'     [strDestName] - name of destination file
'
' PRECONDITION: NewAction() must have already been called
'               for this file copy (of type either
'               gstrKEY_SHAREDFILE or gstrKEY_PRIVATE --
'               see CopySection for an example of how
'               this works).  See NewAction() and related
'               functions in LOGGING.BAS for comments on
'               using the logging function.
'               Either CommitAction() or AbortAction() will
'               allows be called by this procedure, and
'               should not be done by the caller.
'
' Returns: True if copy was successful, False otherwise
'
' POSTCONDITION: The current action will be either committed or
'                aborted.
'-----------------------------------------------------------

Public Function CopyFile(ByVal strSrcDir As String, ByVal strDestDir As String, ByVal strSrcName As String, ByVal strDestName As String, ByVal fShared As Boolean, ByVal fSystem As Boolean) As Boolean
    Const intUNKNOWN% = 0
    Const intCOPIED% = 1
    Const intNOCOPY% = 2
    Const intFILEUPTODATE% = 3

    '
    'VerInstallFile() Flags
    '
    Const VIFF_FORCEINSTALL% = &H1
    Const VIF_TEMPFILE& = &H1
    Const VIF_SRCOLD& = &H4
    Const VIF_DIFFLANG& = &H8
    Const VIF_DIFFCODEPG& = &H10
    Const VIF_DIFFTYPE& = &H20
    Const VIF_WRITEPROT& = &H40
    Const VIF_FILEINUSE& = &H80
    Const VIF_OUTOFSPACE& = &H100
    Const VIF_ACCESSVIOLATION& = &H200
    Const VIF_SHARINGVIOLATION = &H400
    Const VIF_CANNOTCREATE = &H800
    Const VIF_CANNOTDELETE = &H1000
    Const VIF_CANNOTRENAME = &H2000
    Const VIF_OUTOFMEMORY = &H8000&
    Const VIF_CANNOTREADSRC = &H10000
    Const VIF_CANNOTREADDST = &H20000
    Const VIF_BUFFTOOSMALL = &H40000

    Static fIgnoreWarn As Boolean 'user warned about ignoring error?

    Dim strMsg As String
    Dim lRC As Long
    Dim lpTmpNameLen As Long
    Dim intFlags As Integer
    Dim intRESULT As Integer
    Dim fFileAlreadyExisted
    Dim strExt As String
    Dim iMsgFail As Integer, iRet As Integer
    Dim BooAppData As Boolean
    On Error Resume Next

    
    'Ensure that the source file is available for copying
    '
    If Not DetectFile(strSrcDir & strSrcName) Then
        AbortAction
        Exit Function
    End If
      '--make sure path is not too long.
    If Len(strDestDir & strDestName) > MAX_PATH Then
        AbortAction
        strMsg = "Unable to continue. Install path too long. Setup must be run again with shorter path to program folder."
        MsgBox strMsg, 64, sSETUP
        ExitSetup GintRET_FATAL
    End If
    
       MakePath strDestDir, True
       strDestDir = GetLongPath(strDestDir)
       AddSlash strDestDir
         If (Len(sAppDataFol) > 0) Then
           If (InStr(1, strDestDir, sAppDataFol, 1) > 0) Then
              BooAppData = True
           End If
         End If
        
    'Setup for VerInstallFile call
  
    lpTmpNameLen = MAX_PATH
    sVerTmpName = String$(lpTmpNameLen, 0)
    intFlags = VIFF_FORCEINSTALL
    fFileAlreadyExisted = FileExists(strDestDir & strDestName)

    intRESULT = intUNKNOWN

    Do While intRESULT = intUNKNOWN
        'VerInstallFile under Windows 95 does not handle
        '  long filenames, so we must give it the short versions.
        Dim strShortSrcName As String
        Dim strShortDestName As String
        Dim strShortSrcDir As String
        Dim strShortDestDir As String
        Dim nFile As Integer

        strShortSrcName = strSrcName
        strShortSrcDir = strSrcDir
        strShortDestName = strDestName
        strShortDestDir = strDestDir

        On Error GoTo UnexpectedErr
        If (IsNT = False) Then
            If Not FileExists(strDestDir & strDestName) Then
                'If the destination file does not already
                '  exist, we create a dummy with the correct
                '  (long) filename so that we can get its
                '  short filename for VerInstallFile.
                nFile = FreeFile
                Open strDestDir & strDestName For Output Access Write As #nFile
                Close #nFile
            End If
          SeparatePathAndFileName GetShortPathName(strSrcDir & strSrcName), strShortSrcDir, strShortSrcName
          SeparatePathAndFileName GetShortPathName(strDestDir & strDestName), strShortDestDir, strShortDestName
        End If
        On Error Resume Next
         iMsgFail = 0
        lRC = VerInstallFile(intFlags, strShortSrcName, strShortDestName, strShortSrcDir, strShortDestDir, 0&, sVerTmpName, lpTmpNameLen)
          If Err.Number <> 0 Then 'If the version DLL couldn't be found, then abort setup
             ExitSetup GintRET_FATAL
          End If

        If lRC = 0 Then
            '
            'File was successfully installed, increment reference count if needed
            'One more kludge for long filenames: VerInstallFile may have renamed
            'the file to its short version if it went through with the copy.
            'Therefore we simply rename it back to what it should be.
            Name strDestDir & strShortDestName As strDestDir & strDestName
            intRESULT = intCOPIED
        ElseIf lRC And VIF_SRCOLD Then
            'Source file was older, so not copied, the existing version of the file
            'will be used.  Increment reference count if needed
            intRESULT = intFILEUPTODATE
        ElseIf lRC And (VIF_DIFFLANG Or VIF_DIFFCODEPG Or VIF_DIFFTYPE) Then
            'We retry and force installation for these cases.  You can modify the code
            'here to prompt the user about what to do.
            '
            intFlags = VIFF_FORCEINSTALL
        ElseIf lRC And VIF_WRITEPROT Then
            strMsg = "File is write-protected."
            iMsgFail = 1
        ElseIf lRC And VIF_FILEINUSE Then
            strMsg = "File is in use."
            iMsgFail = 1
        ElseIf lRC And VIF_OUTOFSPACE Then
            strMsg = "Not enough disk space."
            iMsgFail = 1
        ElseIf lRC And VIF_ACCESSVIOLATION Then
            strMsg = "Access violation."
            iMsgFail = 1
        ElseIf lRC And VIF_SHARINGVIOLATION Then
            strMsg = "Sharing violation."
            iMsgFail = 1
        ElseIf lRC And VIF_OUTOFMEMORY Then
            strMsg = "Out of memory."
            iMsgFail = 1
        Else
            ' For these cases, we generically report the error and do not install the file
            ' unless this is an SMS install; in which case we abort.
            If lRC And VIF_CANNOTCREATE Then
                iMsgFail = 2
            ElseIf lRC And VIF_CANNOTDELETE Then
                iMsgFail = 2
            ElseIf lRC And VIF_CANNOTRENAME Then
                 iMsgFail = 2
            ElseIf lRC And VIF_CANNOTREADSRC Then
                 iMsgFail = 2
            ElseIf lRC And VIF_CANNOTREADDST Then
                 iMsgFail = 2
            ElseIf lRC And VIF_BUFFTOOSMALL Then
                 iMsgFail = 2
            End If
         End If
         
        If iMsgFail = 1 Then
          strMsg = strMsg & "Unable to install file:" & vbCrLf
          strMsg = strMsg & strDestDir & strDestName
          iRet = MsgBox(strMsg, vbAbortRetryIgnore Or vbExclamation Or vbDefaultButton2, sSETUP)
            Select Case iRet
              Case vbAbort
                ExitSetup GintRET_ABORT
              Case vbIgnore
                intRESULT = intNOCOPY
            End Select
            
        ElseIf iMsgFail = 2 Then
          MsgBox "Unable to install file: " & strDestName, 64, sSETUP
          intRESULT = intNOCOPY
        End If
    Loop

    'If there was a temp file left over from VerInstallFile, remove it
    If lRC And VIF_TEMPFILE Then
        Kill sVerTmpName
    End If

    strExt = GetFileExtension(strDestName)
    'Abort or commit the current Action, and do reference counting
    Select Case intRESULT
        Case intNOCOPY
            AbortAction
        Case intCOPIED
            DecideIncrementRefCount strDestDir & strDestName, fShared, fSystem, fFileAlreadyExisted
            Select Case strExt
                Case GsEXT_FONTFON, GsEXT_FONTTTF 'strExt, gsEXT_FONTFON, and gsEXT_FONTTTF are uppercase
                    'do nothing
                Case Else
                   If (BooAppData = False) Then  '-- avoid logging app data files.
                      AddActionNote "New file copied."
                      CommitAction
                   Else
                      AbortAction '-- don't log files going to app. data folder because they won't be removed.
                   End If
            End Select
            CopyFile = True
        Case intFILEUPTODATE
            'Note: This should never occur - we've already checked to see if the
            '   file was up to date before calling VerInstallFile. But we'll
            '   leave it here for completeness.
            DecideIncrementRefCount strDestDir & strDestName, fShared, fSystem, fFileAlreadyExisted
            Select Case strExt
                Case GsEXT_FONTFON, GsEXT_FONTTTF 'strExt, gsEXT_FONTFON, and gsEXT_FONTTTF are uppercase
                    'do nothing
                Case Else
                  If (BooAppData = False) Then  '-- avoid logging app data files.
                    AddActionNote "Existing file was up to date."
                    CommitAction
                  Else
                    AbortAction
                  End If
            End Select
            CopyFile = True
        Case Else
            AbortAction ' Defensive - this shouldn't be reached
    End Select

    Exit Function

UnexpectedErr:
   MsgBox "Unexpected error: " & Err.Description, 64, sSETUP
   ExitSetup GintRET_FATAL
End Function

Private Function DetectFile(ByVal strFilename As String) As Boolean
    Dim strMsg As String
    Dim iRet As Integer

    DetectFile = True

    Do Until FileExists(strFilename)
        strMsg = "Unable to open file:" & vbLf & vbLf & strFilename
        iRet = MsgBox(strMsg, vbAbortRetryIgnore Or vbExclamation Or vbDefaultButton2, sSETUP)
        Select Case iRet
          Case vbAbort
             ExitSetup GintRET_ABORT
          Case vbIgnore
             DetectFile = False
             Exit Do
        End Select
    Loop
End Function

Public Function SyncShell(CommandLine As String, Optional Timeout As Long, _
    Optional WaitForInputIdle As Boolean, Optional Hide As Boolean = False) As Boolean

    Dim hProcess As Long

    Dim ret As Long
    Dim nMilliseconds As Long

    If Timeout > 0 Then
        nMilliseconds = Timeout
    Else
        nMilliseconds = INFINITE
    End If

    hProcess = StartProcess(CommandLine, Hide)

    If WaitForInputIdle Then
        'Wait for the shelled application to finish setting up its UI:
        ret = InputIdle(hProcess, nMilliseconds)
    Else
        'Wait for the shelled application to terminate:
        ret = WaitForSingleObject(hProcess, nMilliseconds)
    End If

    CloseHandle hProcess

    'Return True if the application finished. Otherwise it timed out or erred.
    SyncShell = (ret = WAIT_OBJECT_0)
End Function

Public Function StartProcess(CommandLine As String, Optional Hide As Boolean = False) As Long
    Const STARTF_USESHOWWINDOW As Long = &H1
    Const SW_HIDE As Long = 0
    
    Dim proc As PROCESS_INFORMATION
    Dim Start As STARTUPINFO

    'Initialize the STARTUPINFO structure:
    Start.cb = Len(Start)
    If Hide Then
        Start.dwFlags = STARTF_USESHOWWINDOW
        Start.wShowWindow = SW_HIDE
    End If
    'Start the shelled application:
    CreateProcessA 0&, CommandLine, 0&, 0&, 1&, _
        NORMAL_PRIORITY_CLASS, 0&, 0&, Start, proc

    StartProcess = proc.hProcess
End Function

Public Function GetFileTime(ByVal aDate As Date) As FILETIME
    Dim lTemp As SYSTEMTIME
    Dim lTime As FILETIME
    
    VariantTimeToSystemTime aDate, lTemp
    SystemTimeToFileTime lTemp, lTime
    LocalFileTimeToFileTime lTime, GetFileTime
End Function

